﻿using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantDataModels.Enums;
using PrecastConcretePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PrecastConcretePlantDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int ClientId { get; set; }
        [Required]
        public int ReinforcedId { get; private set; }
        [Required]
        public int Count { get; private set; }
        public int? ImplementerId { get; set; }

        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public virtual Reinforced Reinforced { get; set; }
        public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ClientId=model.ClientId,
                ReinforcedId = model.ReinforcedId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel
        {
            get
            {
                using var context = new PrecastConcretePlantDatabase();
                return new OrderViewModel
                {
                    Id = Id,
                    ReinforcedId = ReinforcedId,
                    ClientId = ClientId,
                    ImplementerId = ImplementerId,
                    ClientFIO = context.Clients.FirstOrDefault(x => x.Id == ClientId)?.ClientFIO ?? string.Empty,
                    ReinforcedName = context.Reinforceds.FirstOrDefault(x => x.Id == ReinforcedId)?.ReinforcedName ?? string.Empty,
                    Count = Count,
                    Sum = Sum,
                    Status = Status,
                    DateCreate = DateCreate,
                    DateImplement = DateImplement,
                    ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty
                };
            }
        }
    }
}
